@@ -95,7 +95,7 @@ CmdLineParser::CmdLineParser(Settings *settings)
 {
 }
 
-void CmdLineParser::PrintMessage(const std::string &message) const
+void CmdLineParser::PrintMessage(const std::string &message)
 {
     std::cout << message << std::endl;
 }
@@ -121,24 +121,30 @@ bool CmdLineParser::ParseFromArgs(int argc, const char* const argv[])
         else if (strcmp(argv[i], "--debug-fp") == 0)
             _settings->debugFalsePositive = true;
 
-        // Enable all checks - will be removed in future
-        else if (strcmp(argv[i], "-a") == 0 || strcmp(argv[i], "--all") == 0) {
-            PrintMessage("cppcheck: '-a/--all' option is deprecated and will be removed in 1.55 release.");
-            PrintMessage("cppcheck:   please use '--enable=all' instead.");
-        }
-
         // Inconclusive checking (still in testing phase)
         else if (strcmp(argv[i], "--inconclusive") == 0)
             _settings->inconclusive = true;
 
-        // Checking coding style - will be removed in the future
-        else if (strcmp(argv[i], "-s") == 0 || strcmp(argv[i], "--style") == 0) {
-            PrintMessage("cppcheck: '-s/--style' option is deprecated and will be removed in 1.55 release.");
-            PrintMessage("cppcheck:   please use '--enable=style' instead.");
+        // Enforce language (--language=, -x)
+        else if (strncmp(argv[i], "--language=", 11) == 0 || strcmp(argv[i], "-x") == 0) {
+            std::string str;
+            if (argv[i][2]) {
+                str = argv[i]+11;
+            } else {
+                i++;
+                if (i >= argc || argv[i][0] == '-') {
+                    PrintMessage("cppcheck: No language given to '-x' option.");
+                    return false;
+                }
+                str = argv[i];
+            }
 
-            const std::string errmsg = _settings->addEnabled("style");
-            if (!errmsg.empty()) {
-                PrintMessage(errmsg);
+            if (str == "c")
+                _settings->enforcedLang = Settings::C;
+            else if (str == "c++")
+                _settings->enforcedLang = Settings::CPP;
+            else {
+                PrintMessage("cppcheck: Unknown language '" + str + "' enforced.");
                 return false;
             }
         }
@@ -166,7 +172,7 @@ bool CmdLineParser::ParseFromArgs(int argc, const char* const argv[])
 
             std::ifstream f(filename.c_str());
             if (!f.is_open()) {
-                PrintMessage("cppcheck: Couldn't open the file: \"" + std::string(filename) + "\".");
+                PrintMessage("cppcheck: Couldn't open the file: \"" + filename + "\".");
                 return false;
             }
             const std::string errmsg(_settings->nofail.parseFile(f));
@@ -182,7 +188,7 @@ bool CmdLineParser::ParseFromArgs(int argc, const char* const argv[])
             std::ifstream f(filename.c_str());
             if (!f.is_open()) {
                 std::string message("cppcheck: Couldn't open the file: \"");
-                message += std::string(filename);
+                message += filename;
                 message += "\".";
                 if (count(filename.begin(), filename.end(), ',') > 0 ||
                     count(filename.begin(), filename.end(), '.') > 1) {
@@ -249,6 +255,25 @@ bool CmdLineParser::ParseFromArgs(int argc, const char* const argv[])
         else if (strcmp(argv[i], "-f") == 0 || strcmp(argv[i], "--force") == 0)
             _settings->_force = true;
 
+        // Output relative paths
+        else if (strcmp(argv[i], "-rp") == 0 || strcmp(argv[i], "--relative-paths") == 0)
+            _settings->_relativePaths = true;
+        else if (strncmp(argv[i], "-rp=", 4) == 0 || strncmp(argv[i], "--relative-paths=", 17) == 0) {
+            _settings->_relativePaths = true;
+            if (argv[i][argv[i][3]=='='?4:17] != 0) {
+                std::string paths = argv[i]+(argv[i][3]=='='?4:17);
+                std::string::size_type pos;
+                do {
+                    pos = paths.find(';');
+                    _settings->_basePaths.push_back(Path::fromNativeSeparators(paths.substr(0, pos)));
+                    paths.erase(0, pos+1);
+                } while (pos != std::string::npos);
+            } else {
+                PrintMessage("cppcheck: No paths specified for the '" + std::string(argv[i]) + "' option.");
+                return false;
+            }
+        }
+
         // Write results in results.xml
         else if (strcmp(argv[i], "--xml") == 0)
             _settings->_xml = true;
@@ -329,6 +354,14 @@ bool CmdLineParser::ParseFromArgs(int argc, const char* const argv[])
                 define = 2 + argv[i];
             }
 
+            // No "=", append a "=1"
+            if (define.find("=") == std::string::npos)
+                define += "=1";
+
+            // DEF= => empty define
+            else if (define.find("=") + 1U == define.size())
+                define.erase(define.size() - 1U);
+
             if (!_settings->userDefines.empty())
                 _settings->userDefines += ";";
             _settings->userDefines += define;
@@ -433,15 +466,16 @@ bool CmdLineParser::ParseFromArgs(int argc, const char* const argv[])
         // --std
         else if (strcmp(argv[i], "--std=posix") == 0) {
             _settings->standards.posix = true;
-        }
-
-        // --C99
-        else if (strcmp(argv[i], "--std=c99") == 0) {
-            _settings->standards.c99 = true;
-        }
-
-        else if (strcmp(argv[i], "--std=c++11") == 0) {
-            _settings->standards.cpp11 = true;
+        } else if (strcmp(argv[i], "--std=c89") == 0) {
+            _settings->standards.c = Standards::C89;
+        } else if (strcmp(argv[i], "--std=c99") == 0) {
+            _settings->standards.c = Standards::C99;
+        } else if (strcmp(argv[i], "--std=c11") == 0) {
+            _settings->standards.c = Standards::C11;
+        } else if (strcmp(argv[i], "--std=c++03") == 0) {
+            _settings->standards.cpp = Standards::CPP03;
+        } else if (strcmp(argv[i], "--std=c++11") == 0) {
+            _settings->standards.cpp = Standards::CPP11;
         }
 
         // Output formatter
@@ -499,12 +533,6 @@ bool CmdLineParser::ParseFromArgs(int argc, const char* const argv[])
             }
         }
 
-        // deprecated: auto deallocated classes..
-        else if (strcmp(argv[i], "--auto-dealloc") == 0) {
-            ++i;
-            PrintMessage("cppcheck: '--auto-dealloc' option is deprecated and will be removed in 1.55 release.");
-        }
-
         // print all possible error messages..
         else if (strcmp(argv[i], "--errorlist") == 0) {
             _showErrorMessages = true;
@@ -517,27 +545,18 @@ bool CmdLineParser::ParseFromArgs(int argc, const char* const argv[])
             std::ostringstream doc;
             // Get documentation..
             for (std::list<Check *>::iterator it = Check::instances().begin(); it != Check::instances().end(); ++it) {
-                const std::string name((*it)->name());
+                const std::string& name((*it)->name());
                 const std::string info((*it)->classInfo());
                 if (!name.empty() && !info.empty())
                     doc << "===" << name << "===\n"
-                        << info << "\n\n";
+                        << info << "\n";
             }
 
-            std::string doc2(doc.str());
-            while (doc2.find("\n\n\n") != std::string::npos)
-                doc2.erase(doc2.find("\n\n\n"), 1);
-            std::cout << doc2;
+            std::cout << doc.str();
             _exitAfterPrint = true;
             return true;
         }
 
-        // --test-2-pass Experimental 2-pass checking of files
-        // This command line flag will be removed
-        else if (strcmp(argv[i], "--test-2-pass") == 0) {
-            _settings->test_2_pass = true;
-        }
-
         // show timing information..
         else if (strncmp(argv[i], "--showtime=", 11) == 0) {
             const std::string showtimeMode = argv[i] + 11;
@@ -615,7 +634,7 @@ bool CmdLineParser::ParseFromArgs(int argc, const char* const argv[])
                 _settings->platform(Settings::Unix64);
             else {
                 std::string message("cppcheck: error: unrecognized platform: \"");
-                message += argv[i];
+                message += platform;
                 message +=  "\".";
                 PrintMessage(message);
                 return false;
@@ -665,11 +684,6 @@ bool CmdLineParser::ParseFromArgs(int argc, const char* const argv[])
         PrintMessage("cppcheck: unusedFunction check can't be used with '-j' option, so it's disabled.");
     }
 
-    // FIXME: Make the _settings.test_2_pass thread safe
-    if (_settings->test_2_pass && _settings->_jobs > 1) {
-        PrintMessage("cppcheck: --test-2-pass doesn't work with -j option yet.");
-    }
-
     if (argc <= 1)
         _showHelp = true;
 
@@ -684,10 +698,14 @@ bool CmdLineParser::ParseFromArgs(int argc, const char* const argv[])
         return false;
     }
 
+    // Use paths _pathnames if no base paths for relative path output are given
+    if (_settings->_basePaths.empty() && _settings->_relativePaths)
+        _settings->_basePaths = _pathnames;
+
     return true;
 }
 
-void CmdLineParser::PrintHelp() const
+void CmdLineParser::PrintHelp()
 {
     std::cout <<   "Cppcheck - A tool for static C/C++ code analysis\n"
               "\n"
@@ -741,7 +759,7 @@ void CmdLineParser::PrintHelp() const
               "                         filename per line. When file is '-,' the file list will\n"
               "                         be read from standard input.\n"
               "    -f, --force          Force checking of all configurations in files. If used\n"
-              "                         together with '--max-ifdefs=', the last option is the\n"
+              "                         together with '--max-configs=', the last option is the\n"
               "                         one that is effective.\n"
               "    -h, --help           Print this help.\n"
               "    -I <dir>             Give path to search for include files. Give several -I\n"
@@ -758,10 +776,18 @@ void CmdLineParser::PrintHelp() const
               "                         from the check. This applies only to source files so\n"
               "                         header files included by source files are not matched.\n"
               "                         Directory name is matched to all parts of the path.\n"
+              "    --inconclusive       Allow that Cppcheck reports even though the analysis is\n"
+              "                         inconclusive.\n"
+              "                         There are false positives with this option. Each result\n"
+              "                         must be carefully investigated before you know if it is\n"
+              "                         good or bad.\n"
               "    --inline-suppr       Enable inline suppressions. Use them by placing one or\n"
               "                         more comments, like: '// cppcheck-suppress warningId'\n"
               "                         on the lines before the warning to suppress.\n"
               "    -j <jobs>            Start [jobs] threads to do the checking simultaneously.\n"
+              "    --language=<language>, -x <language>\n"
+              "                         Forces cppcheck to check all files as the given\n"
+              "                         language. Valid values are: c, c++\n"
               "    --max-configs=<limit>\n"
               "                         Maximum number of configurations to check in a file\n"
               "                         before skipping it. Default is '12'. If used together\n"
@@ -780,23 +806,37 @@ void CmdLineParser::PrintHelp() const
               "                          * win64\n"
               "                                 64 bit Windows\n"
               "    -q, --quiet          Only print error messages.\n"
+              "    -rp, --relative-paths\n"
+              "    -rp=<paths>, --relative-paths=<paths>\n"
+              "                         Use relative paths in output. When given, <paths> are\n"
+              "                         used as base. You can separate multiple paths by ';'.\n"
+              "                         Otherwise path where source files are searched is used.\n"
+              "                         We use string comparison to create relative paths, so\n"
+              "                         using e.g. ~ for home folder does not work. It is\n"
+              "                         currently only possible to apply the base paths to\n"
+              "                         files that are on a lower level in the directory tree.\n"
               "    --report-progress    Report progress messages while checking a file.\n"
 #ifdef HAVE_RULES
               "    --rule=<rule>        Match regular expression.\n"
               "    --rule-file=<file>   Use given rule file. For more information, see: \n"
               "                         https://sourceforge.net/projects/cppcheck/files/Articles/\n"
 #endif
-              "    -s, --style          Deprecated, please use '--enable=style' instead\n"
-              "    --std=<id>           Enable some standard related checks.\n"
+              "    --std=<id>           Set standard.\n"
               "                         The available options are:\n"
               "                          * posix\n"
-              "                                 Checks related to POSIX-specific functionality\n"
+              "                                 POSIX compatible code\n"
+              "                          * c89\n"
+              "                                 C code is C89 compatible\n"
               "                          * c99\n"
-              "                                 C99 standard related checks\n"
+              "                                 C code is C99 compatible\n"
+              "                          * c11\n"
+              "                                 C code is C11 compatible (default)\n"
+              "                          * c++03\n"
+              "                                 C++ code is C++03 compatible\n"
               "                          * c++11\n"
-              "                                 C++11 standard related checks\n"
-              "                         Example to enable more than one checks:\n"
-              "                           'cppcheck --std=c99 --std=posix file.cpp'\n"
+              "                                 C++ code is C++11 compatible (default)\n"
+              "                         More than one --std can be used:\n"
+              "                           'cppcheck --std=c99 --std=posix file.c'\n"
               "    --suppress=<spec>    Suppress warnings that match <spec>. The format of\n"
               "                         <spec> is:\n"
               "                         [error id]:[filename]:[line]\n"
@@ -818,14 +858,17 @@ void CmdLineParser::PrintHelp() const
               "\n"
               "Example usage:\n"
               "  # Recursively check the current folder. Print the progress on the screen and\n"
-              "    write errors to a file:\n"
-              "    cppcheck . 2> err.txt\n"
+              "  # write errors to a file:\n"
+              "  cppcheck . 2> err.txt\n"
+              "\n"
               "  # Recursively check ../myproject/ and don't print progress:\n"
-              "    cppcheck --quiet ../myproject/\n"
-              "  # Check only files one.cpp and two.cpp and give all information there is:\n"
-              "    cppcheck -v -s one.cpp two.cpp\n"
+              "  cppcheck --quiet ../myproject/\n"
+              "\n"
+              "  # Check test.cpp, enable all checks:\n"
+              "  cppcheck --enable=all --inconclusive --std=posix test.cpp\n"
+              "\n"
               "  # Check f.cpp and search include files from inc1/ and inc2/:\n"
-              "    cppcheck -I inc1/ -I inc2/ f.cpp\n"
+              "  cppcheck -I inc1/ -I inc2/ f.cpp\n"
               "\n"
               "For more information:\n"
               "    http://cppcheck.sf.net/manual.pdf\n";
