@@ -22,11 +22,13 @@
 #define CheckOtherH
 //---------------------------------------------------------------------------
 
+#include "config.h"
 #include "check.h"
 #include "settings.h"
 
 class Token;
 class Function;
+class Variable;
 
 /// @addtogroup Checks
 /// @{
@@ -34,7 +36,7 @@ class Function;
 
 /** @brief Various small checks */
 
-class CheckOther : public Check {
+class CPPCHECKLIB CheckOther : public Check {
 public:
     /** @brief This constructor is used when registering the CheckClass */
     CheckOther() : Check(myName())
@@ -49,17 +51,19 @@ public:
     void runChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) {
         CheckOther checkOther(tokenizer, settings, errorLogger);
 
-        // Coding style checks
+        // Checks
         checkOther.warningOldStylePointerCast();
+        checkOther.invalidPointerCast();
         checkOther.checkUnsignedDivision();
         checkOther.checkCharVariable();
         checkOther.strPlusChar();
         checkOther.sizeofsizeof();
         checkOther.sizeofCalculation();
+        checkOther.checkRedundantAssignment();
         checkOther.checkRedundantAssignmentInSwitch();
         checkOther.checkAssignmentInAssert();
         checkOther.checkSizeofForArrayParameter();
-        checkOther.checkSizeofForStrncmpSize();
+        checkOther.checkSizeofForPointerSize();
         checkOther.checkSizeofForNumericParameter();
         checkOther.checkSelfAssignment();
         checkOther.checkDuplicateIf();
@@ -67,22 +71,22 @@ public:
         checkOther.checkDuplicateExpression();
         checkOther.checkUnreachableCode();
         checkOther.checkSuspiciousSemicolon();
-        checkOther.checkWrongPrintfScanfArguments();
-
-        // information checks
         checkOther.checkVariableScope();
-
         checkOther.clarifyCondition();   // not simplified because ifAssign
         checkOther.checkComparisonOfBoolExpressionWithInt();
+        checkOther.checkSignOfUnsignedVariable();  // don't ignore casts (#3574)
+        checkOther.checkIncompleteArrayFill();
+        checkOther.checkSuspiciousStringCompare();
     }
 
     /** @brief Run checks against the simplified token list */
     void runSimplifiedChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) {
         CheckOther checkOther(tokenizer, settings, errorLogger);
 
+        // Checks
+        checkOther.oppositeInnerCondition();
         checkOther.clarifyCalculation();
-
-        // Coding style checks
+        checkOther.clarifyStatement();
         checkOther.checkConstantFunctionParameter();
         checkOther.checkIncompleteStatement();
 
@@ -90,36 +94,45 @@ public:
         checkOther.checkZeroDivision();
         checkOther.checkMathFunctions();
         checkOther.checkCCTypeFunctions();
-        checkOther.checkFflushOnInputStream();
-        checkOther.invalidScanf();
 
-        checkOther.checkCoutCerrMisusage();
         checkOther.checkIncorrectLogicOperator();
         checkOther.checkMisusedScopedObject();
+        checkOther.checkComparisonOfFuncReturningBool();
+        checkOther.checkComparisonOfBoolWithBool();
         checkOther.checkMemsetZeroBytes();
         checkOther.checkIncorrectStringCompare();
         checkOther.checkIncrementBoolean();
         checkOther.checkComparisonOfBoolWithInt();
         checkOther.checkSwitchCaseFallThrough();
         checkOther.checkAlwaysTrueOrFalseStringCompare();
+        checkOther.checkModuloAlwaysTrueFalse();
 
         checkOther.checkAssignBoolToPointer();
-        checkOther.checkSignOfUnsignedVariable();
         checkOther.checkBitwiseOnBoolean();
+        checkOther.checkInvalidFree();
         checkOther.checkDoubleFree();
+        checkOther.checkRedundantCopy();
+        checkOther.checkNegativeBitwiseShift();
     }
 
+    /** To check the dead code in a program, which is unaccessible due to the counter-conditions check in nested-if statements **/
+    void oppositeInnerCondition();
+
     /** @brief Clarify calculation for ".. a * b ? .." */
     void clarifyCalculation();
-    void clarifyCalculationError(const Token *tok, const std::string &op);
 
     /** @brief Suspicious condition (assignment+comparison) */
     void clarifyCondition();
-    void clarifyConditionError(const Token *tok, bool assign, bool boolop);
+
+    /** @brief Suspicious statement like '*A++;' */
+    void clarifyStatement();
 
     /** @brief Are there C-style pointer casts in a c++ file? */
     void warningOldStylePointerCast();
 
+    /** @brief Check for pointer casts to a type with an incompatible binary data representation */
+    void invalidPointerCast();
+
     /**
      * @brief Invalid function usage (invalid radix / overlapping data)
      *
@@ -134,6 +147,7 @@ public:
 
     /** @brief %Check scope of variables */
     void checkVariableScope();
+    void lookupVar(const Token *tok, const Variable* var);
 
     /** @brief %Check for constant function parameter */
     void checkConstantFunctionParameter();
@@ -156,35 +170,14 @@ public:
     /** @brief %Check for parameters given to cctype function that do make error*/
     void checkCCTypeFunctions();
 
-    void lookupVar(const Token *tok1, const std::string &varname);
-
-    /** @brief %Check for using fflush() on an input stream*/
-    void checkFflushOnInputStream();
-
     /** @brief %Check for 'sizeof sizeof ..' */
     void sizeofsizeof();
-    void sizeofsizeofError(const Token *tok);
 
     /** @brief %Check for calculations inside sizeof */
     void sizeofCalculation();
-    void sizeofCalculationError(const Token *tok);
 
-    /** @brief scanf can crash if width specifiers are not used */
-    void invalidScanf();
-    void invalidScanfError(const Token *tok);
-
-    /** @brief %Checks type and number of arguments given to functions like printf or scanf*/
-    void checkWrongPrintfScanfArguments();
-    void wrongPrintfScanfArgumentsError(const Token* tok,
-                                        const std::string &function,
-                                        unsigned int numFormat,
-                                        unsigned int numFunction);
-    void invalidScanfArgTypeError(const Token* tok, const std::string &functionName, unsigned int numFormat);
-    void invalidPrintfArgTypeError_s(const Token* tok, unsigned int numFormat);
-    void invalidPrintfArgTypeError_n(const Token* tok, unsigned int numFormat);
-    void invalidPrintfArgTypeError_p(const Token* tok, unsigned int numFormat);
-    void invalidPrintfArgTypeError_int(const Token* tok, unsigned int numFormat, char c);
-    void invalidPrintfArgTypeError_float(const Token* tok, unsigned int numFormat, char c);
+    /** @brief copying to memory or assigning to a variablen twice */
+    void checkRedundantAssignment();
 
     /** @brief %Check for assigning to the same variable twice in a switch statement*/
     void checkRedundantAssignmentInSwitch();
@@ -192,9 +185,6 @@ public:
     /** @brief %Check for switch case fall through without comment */
     void checkSwitchCaseFallThrough();
 
-    /** @brief %Check for missusage of std::cout */
-    void checkCoutCerrMisusage();
-
     /** @brief %Check for assigning a variable to itself*/
     void checkSelfAssignment();
 
@@ -207,14 +197,20 @@ public:
     /** @brief %Check for objects that are destroyed immediately */
     void checkMisusedScopedObject();
 
+    /** @brief %Check for comparison of function returning bool*/
+    void checkComparisonOfFuncReturningBool();
+
+    /** @brief %Check for comparison of variable of type bool*/
+    void checkComparisonOfBoolWithBool();
+
     /** @brief %Check for filling zero bytes with memset() */
     void checkMemsetZeroBytes();
 
     /** @brief %Check for using sizeof with array given as function argument */
     void checkSizeofForArrayParameter();
 
-    /** @brief %Check for using sizeof with a char pointer */
-    void checkSizeofForStrncmpSize();
+    /** @brief %Check for using sizeof of a variable when allocating it */
+    void checkSizeofForPointerSize();
 
     /** @brief %Check for using sizeof with numeric given as function argument */
     void checkSizeofForNumericParameter();
@@ -222,6 +218,9 @@ public:
     /** @brief %Check for using bad usage of strncmp and substr */
     void checkIncorrectStringCompare();
 
+    /** @brief %Check for comparison of a string literal with a char* variable */
+    void checkSuspiciousStringCompare();
+
     /** @brief %Check for using postfix increment on bool */
     void checkIncrementBoolean();
 
@@ -240,6 +239,9 @@ public:
     /** @brief %Check for suspicious code that compares string literals for equality */
     void checkAlwaysTrueOrFalseStringCompare();
 
+    /** @brief %Check for suspicious usage of modulo (e.g. "if(var % 4 == 4)") */
+    void checkModuloAlwaysTrueFalse();
+
     /** @brief %Check for code that gets never executed, such as duplicate break statements */
     void checkUnreachableCode();
 
@@ -258,12 +260,34 @@ public:
     /** @brief %Check for suspicious use of semicolon */
     void checkSuspiciousSemicolon();
 
+    /** @brief %Check for free() operations on invalid memory locations */
+    void checkInvalidFree();
+    void invalidFreeError(const Token *tok);
+
     /** @brief %Check for double free or double close operations */
     void checkDoubleFree();
+    void doubleFreeError(const Token *tok, const std::string &varname);
 
+    /** @brief %Check for code creating redundant copies */
+    void checkRedundantCopy();
+
+    /** @brief %Check for bitwise operation with negative right operand */
+    void checkNegativeBitwiseShift();
+
+    /** @brief %Check for buffers that are filled incompletely with memset and similar functions */
+    void checkIncompleteArrayFill();
+
+private:
     // Error messages..
+    void oppositeInnerConditionError(const Token *tok);
+    void clarifyCalculationError(const Token *tok, const std::string &op);
+    void clarifyConditionError(const Token *tok, bool assign, bool boolop);
+    void clarifyStatementError(const Token* tok);
+    void sizeofsizeofError(const Token *tok);
+    void sizeofCalculationError(const Token *tok, bool inconclusive);
     void cstyleCastError(const Token *tok);
-    void dangerousUsageStrtolError(const Token *tok);
+    void invalidPointerCastError(const Token* tok, const std::string& from, const std::string& to, bool inconclusive);
+    void dangerousUsageStrtolError(const Token *tok, const std::string& funcname);
     void sprintfOverlappingDataError(const Token *tok, const std::string &varname);
     void udivError(const Token *tok, bool inconclusive);
     void passedByValueError(const Token *tok, const std::string &parname);
@@ -273,43 +297,54 @@ public:
     void variableScopeError(const Token *tok, const std::string &varname);
     void strPlusCharError(const Token *tok);
     void zerodivError(const Token *tok);
-    void coutCerrMisusageError(const Token* tok, const std::string& streamName);
     void mathfunctionCallError(const Token *tok, const unsigned int numParam = 1);
     void cctypefunctionCallError(const Token *tok, const std::string &functionName, const std::string &value);
-    void fflushOnInputStreamError(const Token *tok, const std::string &varname);
-    void redundantAssignmentInSwitchError(const Token *tok, const std::string &varname);
-    void redundantStrcpyInSwitchError(const Token *tok, const std::string &varname);
+    void redundantAssignmentError(const Token *tok1, const Token* tok2, const std::string& var);
+    void redundantAssignmentInSwitchError(const Token *tok1, const Token *tok2, const std::string &var);
+    void redundantCopyError(const Token *tok1, const Token* tok2, const std::string& var);
+    void redundantCopyInSwitchError(const Token *tok1, const Token* tok2, const std::string &var);
+    void redundantBitwiseOperationInSwitchError(const Token *tok, const std::string &varname);
     void switchCaseFallThrough(const Token *tok);
     void selfAssignmentError(const Token *tok, const std::string &varname);
     void assignmentInAssertError(const Token *tok, const std::string &varname);
-    void incorrectLogicOperatorError(const Token *tok, bool always);
-    void secondAlwaysTrueFalseWhenFirstTrueError(const Token *tok, const std::string &truefalse);
+    void incorrectLogicOperatorError(const Token *tok, const std::string &condition, bool always);
+    void redundantConditionError(const Token *tok, const std::string &text);
     void misusedScopeObjectError(const Token *tok, const std::string &varname);
+    void comparisonOfFuncReturningBoolError(const Token *tok, const std::string &expression);
+    void comparisonOfTwoFuncsReturningBoolError(const Token *tok, const std::string &expression1, const std::string &expression2);
+    void comparisonOfBoolWithBoolError(const Token *tok, const std::string &expression);
     void memsetZeroBytesError(const Token *tok, const std::string &varname);
     void sizeofForArrayParameterError(const Token *tok);
-    void sizeofForStrncmpError(const Token *tok);
+    void sizeofForPointerError(const Token *tok, const std::string &varname);
     void sizeofForNumericParameterError(const Token *tok);
-    void incorrectStringCompareError(const Token *tok, const std::string& func, const std::string &string, const std::string &len);
+    void incorrectStringCompareError(const Token *tok, const std::string& func, const std::string &string);
     void incorrectStringBooleanError(const Token *tok, const std::string& string);
     void incrementBooleanError(const Token *tok);
-    void comparisonOfBoolWithIntError(const Token *tok, const std::string &expression);
+    void comparisonOfBoolWithIntError(const Token *tok, const std::string &expression, bool n0o1);
+    void comparisonOfBoolWithInvalidComparator(const Token *tok, const std::string &expression);
     void duplicateIfError(const Token *tok1, const Token *tok2);
     void duplicateBranchError(const Token *tok1, const Token *tok2);
     void duplicateExpressionError(const Token *tok1, const Token *tok2, const std::string &op);
     void alwaysTrueFalseStringCompareError(const Token *tok, const std::string& str1, const std::string& str2);
     void alwaysTrueStringVariableCompareError(const Token *tok, const std::string& str1, const std::string& str2);
-    void duplicateBreakError(const Token *tok);
-    void unreachableCodeError(const Token* tok);
+    void suspiciousStringCompareError(const Token* tok, const std::string& var);
+    void duplicateBreakError(const Token *tok, bool inconclusive);
+    void unreachableCodeError(const Token* tok, bool inconclusive);
     void assignBoolToPointerError(const Token *tok);
     void unsignedLessThanZeroError(const Token *tok, const std::string &varname, bool inconclusive);
+    void pointerLessThanZeroError(const Token *tok, bool inconclusive);
     void unsignedPositiveError(const Token *tok, const std::string &varname, bool inconclusive);
+    void pointerPositiveError(const Token *tok, bool inconclusive);
     void bitwiseOnBooleanError(const Token *tok, const std::string &varname, const std::string &op);
-    void comparisonOfBoolExpressionWithIntError(const Token *tok);
+    void comparisonOfBoolExpressionWithIntError(const Token *tok, bool n0o1);
     void SuspiciousSemicolonError(const Token *tok);
-    void doubleFreeError(const Token *tok, const std::string &varname);
     void doubleCloseDirError(const Token *tok, const std::string &varname);
+    void moduloAlwaysTrueFalseError(const Token* tok, const std::string& maxVal);
+    void negativeBitwiseShiftError(const Token *tok);
+    void redundantCopyError(const Token *tok, const std::string &varname);
+    void incompleteArrayFillError(const Token* tok, const std::string& buffer, const std::string& function, bool boolean);
 
-    void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) {
+    void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const {
         CheckOther c(0, settings, errorLogger);
 
         // error
@@ -318,17 +353,26 @@ public:
         c.udivError(0, false);
         c.zerodivError(0);
         c.mathfunctionCallError(0);
-        c.fflushOnInputStreamError(0, "stdin");
         c.misusedScopeObjectError(NULL, "varname");
+        c.comparisonOfFuncReturningBoolError(0, "func_name");
+        c.comparisonOfTwoFuncsReturningBoolError(0, "func_name1", "func_name2");
+        c.comparisonOfBoolWithBoolError(0, "var_name");
         c.sizeofForArrayParameterError(0);
-        c.sizeofForStrncmpError(0);
+        c.sizeofForPointerError(0, "varname");
         c.sizeofForNumericParameterError(0);
-        c.coutCerrMisusageError(0, "cout");
         c.doubleFreeError(0, "varname");
+        c.invalidPointerCastError(0, "float", "double", false);
+        c.negativeBitwiseShiftError(0);
+
+        //performance
+        c.redundantCopyError(0, "varname");
+        c.redundantCopyError(0, 0, "var");
+        c.redundantAssignmentError(0, 0, "var");
 
         // style/warning
+        c.oppositeInnerConditionError(0);
         c.cstyleCastError(0);
-        c.dangerousUsageStrtolError(0);
+        c.dangerousUsageStrtolError(0, "strtol");
         c.passedByValueError(0, "parametername");
         c.constStatementError(0, "type");
         c.charArrayIndexError(0);
@@ -336,44 +380,43 @@ public:
         c.variableScopeError(0, "varname");
         c.strPlusCharError(0);
         c.sizeofsizeofError(0);
-        c.sizeofCalculationError(0);
-        c.redundantAssignmentInSwitchError(0, "varname");
+        c.sizeofCalculationError(0, false);
+        c.redundantAssignmentInSwitchError(0, 0, "var");
+        c.redundantCopyInSwitchError(0, 0, "var");
         c.switchCaseFallThrough(0);
         c.selfAssignmentError(0, "varname");
         c.assignmentInAssertError(0, "varname");
-        c.invalidScanfError(0);
-        c.incorrectLogicOperatorError(0, true);
-        c.secondAlwaysTrueFalseWhenFirstTrueError(0, "when first comparison is true, the 2nd comparison is always true");
+        c.incorrectLogicOperatorError(0, "foo > 3 && foo < 4", true);
+        c.redundantConditionError(0, "If x > 10 the condition x > 11 is always true.");
         c.memsetZeroBytesError(0, "varname");
         c.clarifyCalculationError(0, "+");
         c.clarifyConditionError(0, true, false);
-        c.incorrectStringCompareError(0, "substr", "\"Hello World\"", "12");
+        c.clarifyStatementError(0);
+        c.incorrectStringCompareError(0, "substr", "\"Hello World\"");
+        c.suspiciousStringCompareError(0, "foo");
         c.incorrectStringBooleanError(0, "\"Hello World\"");
         c.incrementBooleanError(0);
-        c.comparisonOfBoolWithIntError(0, "varname");
+        c.comparisonOfBoolWithIntError(0, "varname", true);
         c.duplicateIfError(0, 0);
         c.duplicateBranchError(0, 0);
         c.duplicateExpressionError(0, 0, "&&");
         c.alwaysTrueFalseStringCompareError(0, "str1", "str2");
         c.alwaysTrueStringVariableCompareError(0, "varname1", "varname2");
-        c.duplicateBreakError(0);
-        c.unreachableCodeError(0);
+        c.duplicateBreakError(0, false);
+        c.unreachableCodeError(0, false);
         c.unsignedLessThanZeroError(0, "varname", false);
         c.unsignedPositiveError(0, "varname", false);
+        c.pointerLessThanZeroError(0, false);
+        c.pointerPositiveError(0, false);
         c.bitwiseOnBooleanError(0, "varname", "&&");
-        c.comparisonOfBoolExpressionWithIntError(0);
+        c.comparisonOfBoolExpressionWithIntError(0, true);
         c.SuspiciousSemicolonError(0);
-        c.wrongPrintfScanfArgumentsError(0,"printf",3,2);
-        c.invalidScanfArgTypeError(0, "scanf", 1);
-        c.invalidPrintfArgTypeError_s(0, 1);
-        c.invalidPrintfArgTypeError_n(0, 1);
-        c.invalidPrintfArgTypeError_p(0, 1);
-        c.invalidPrintfArgTypeError_int(0, 1, 'u');
-        c.invalidPrintfArgTypeError_float(0, 1, 'f');
         c.cctypefunctionCallError(0, "funname", "value");
+        c.moduloAlwaysTrueFalseError(0, "1");
+        c.incompleteArrayFillError(0, "buffer", "memset", false);
     }
 
-    std::string myName() const {
+    static std::string myName() {
         return "Other";
     }
 
@@ -382,24 +425,28 @@ public:
 
                // error
                "* Assigning bool value to pointer (converting bool value to address)\n"
-               "* [[OverlappingData|bad usage of the function 'sprintf' (overlapping data)]]\n"
                "* division with zero\n"
-               "* using fflush() on an input stream\n"
                "* scoped object destroyed immediately after construction\n"
                "* assignment in an assert statement\n"
                "* sizeof for array given as function argument\n"
                "* sizeof for numeric given as function argument\n"
+               "* using sizeof(pointer) instead of the size of pointed data\n"
                "* incorrect length arguments for 'substr' and 'strncmp'\n"
-               "* invalid usage of output stream. For example: std::cout << std::cout;'\n"
-               "* wrong number of arguments given to 'printf' or 'scanf;'\n"
+               "* free() or delete of an invalid memory location\n"
                "* double free() or double closedir()\n"
+               "* bitwise operation with negative right operand\n"
+
+               //performance
+               "* redundant data copying for const variable\n"
+               "* subsequent assignment or copying to a variable or buffer\n"
 
                // style
+               "* Find dead code which is unaccessible due to the counter-conditions check in nested if statements\n"
                "* C-style pointer cast in cpp file\n"
+               "* casting between incompatible pointer types\n"
                "* redundant if\n"
                "* bad usage of the function 'strtol'\n"
                "* [[CheckUnsignedDivision|unsigned division]]\n"
-               "* Dangerous usage of 'scanf'\n"
                "* passing parameter by value\n"
                "* [[IncompleteStatement|Incomplete statement]]\n"
                "* [[charvar|check how signed char variables are used]]\n"
@@ -407,6 +454,8 @@ public:
                "* condition that is always true/false\n"
                "* unusal pointer arithmetic. For example: \"abc\" + 'd'\n"
                "* redundant assignment in a switch statement\n"
+               "* redundant pre/post operation in a switch statement\n"
+               "* redundant bitwise operation in a switch statement\n"
                "* redundant strcpy in a switch statement\n"
                "* look for 'sizeof sizeof ..'\n"
                "* look for calculations inside sizeof()\n"
@@ -416,9 +465,12 @@ public:
                "* using increment on boolean\n"
                "* comparison of a boolean with a non-zero integer\n"
                "* comparison of a boolean expression with an integer other than 0 or 1\n"
+               "* comparison of a function returning boolean value using relational operator\n"
+               "* comparison of a boolean value with boolean value using relational operator\n"
                "* suspicious condition (assignment+comparison)\n"
                "* suspicious condition (runtime comparison of string literals)\n"
                "* suspicious condition (string literals as boolean)\n"
+               "* suspicious comparison of a string literal with a char* variable\n"
                "* duplicate break statement\n"
                "* unreachable code\n"
                "* testing if unsigned variable is negative\n"
@@ -426,37 +478,16 @@ public:
                "* using bool in bitwise expression\n"
                "* Suspicious use of ; at the end of 'if/for/while' statement.\n"
                "* incorrect usage of functions from ctype library.\n"
-
-               // optimisations
-               "* optimisation: detect post increment/decrement\n";
+               "* Comparisons of modulo results that are always true/false.\n"
+               "* Array filled incompletely using memset/memcpy/memmove.\n";
     }
 
-private:
-
-    /**
-     * @brief Used in warningRedundantCode()
-     * Iterates through the %var% tokens in a fully qualified name and concatenates them.
-     */
-    std::string concatNames(const Token **tok) const {
-        std::string varname;
-        while (Token::Match(*tok, "%var% ::|.")) {
-            varname.append((*tok)->str());
-            varname.append((*tok)->next()->str());
-            *tok = (*tok)->tokAt(2);
-        }
-
-        if (Token::Match(*tok, "%var%"))
-            varname.append((*tok)->str());
-
-        return varname;
-    }
-
-    void checkExpressionRange(const std::list<Function> &constFunctions,
+    void checkExpressionRange(const std::list<const Function*> &constFunctions,
                               const Token *start,
                               const Token *end,
                               const std::string &toCheck);
 
-    void complexDuplicateExpressionCheck(const std::list<Function> &constFunctions,
+    void complexDuplicateExpressionCheck(const std::list<const Function*> &constFunctions,
                                          const Token *classStart,
                                          const std::string &toCheck,
                                          const std::string &alt);
@@ -464,4 +495,3 @@ private:
 /// @}
 //---------------------------------------------------------------------------
 #endif
-
