@@ -33,6 +33,16 @@ namespace {
 
 //---------------------------------------------------------------------------
 
+
+// Skip [ .. ]
+static const Token * skipBrackets(const Token *tok)
+{
+    while (tok && tok->str() == "[")
+        tok = tok->link()->next();
+    return tok;
+}
+
+
 /// @addtogroup Checks
 /// @{
 
@@ -42,8 +52,8 @@ namespace {
 class UninitVar : public ExecutionPath {
 public:
     /** Startup constructor */
-    explicit UninitVar(Check *c)
-        : ExecutionPath(c, 0), pointer(false), array(false), alloc(false), strncpy_(false), memset_nonzero(false) {
+    explicit UninitVar(Check *c, const SymbolDatabase* db, bool isc)
+        : ExecutionPath(c, 0), symbolDatabase(db), isC(isc), var(0), alloc(false), strncpy_(false), memset_nonzero(false) {
     }
 
 private:
@@ -56,24 +66,23 @@ private:
     void operator=(const UninitVar &);
 
     /** internal constructor for creating extra checks */
-    UninitVar(Check *c, unsigned int v, const std::string &name, bool p, bool a)
-        : ExecutionPath(c, v), varname(name), pointer(p), array(a), alloc(false), strncpy_(false), memset_nonzero(false) {
+    UninitVar(Check *c, const Variable* v, const SymbolDatabase* db, bool isc)
+        : ExecutionPath(c, v->varId()), symbolDatabase(db), isC(isc), var(v), alloc(false), strncpy_(false), memset_nonzero(false) {
     }
 
     /** is other execution path equal? */
     bool is_equal(const ExecutionPath *e) const {
         const UninitVar *c = static_cast<const UninitVar *>(e);
-        return (varname == c->varname && pointer == c->pointer && array == c->array && alloc == c->alloc && strncpy_ == c->strncpy_ && memset_nonzero == c->memset_nonzero);
+        return (var == c->var && alloc == c->alloc && strncpy_ == c->strncpy_ && memset_nonzero == c->memset_nonzero);
     }
 
-    /** variable name for this check */
-    const std::string varname;
+    /** pointer to symbol database */
+    const SymbolDatabase* symbolDatabase;
 
-    /** is this variable a pointer? */
-    const bool pointer;
+    const bool isC;
 
-    /** is this variable an array? */
-    const bool array;
+    /** variable for this check */
+    const Variable* var;
 
     /** is this variable allocated? */
     bool  alloc;
@@ -91,8 +100,13 @@ private:
         std::list<ExecutionPath *>::const_iterator it;
         for (it = checks.begin(); it != checks.end(); ++it) {
             UninitVar *c = dynamic_cast<UninitVar *>(*it);
-            if (c && c->varId == varid)
-                c->alloc = true;
+            if (c && c->varId == varid) {
+                if (c->var->isPointer() && !c->var->isArray())
+                    c->alloc = true;
+                else
+                    bailOutVar(checks, varid);
+                break;
+            }
         }
     }
 
@@ -108,7 +122,7 @@ private:
         while (it != checks.end()) {
             UninitVar *c = dynamic_cast<UninitVar *>(*it);
             if (c && c->varId == varid) {
-                if (c->alloc || c->array) {
+                if (c->alloc || c->var->isArray()) {
                     delete c;
                     checks.erase(it++);
                     continue;
@@ -134,10 +148,10 @@ private:
             UninitVar *c = dynamic_cast<UninitVar *>(*it);
             if (c && c->varId == varid) {
                 // unallocated pointer variable => error
-                if (c->pointer && !c->alloc) {
+                if (c->var->isPointer() && !c->var->isArray() && !c->alloc) {
                     CheckUninitVar *checkUninitVar = dynamic_cast<CheckUninitVar *>(c->owner);
                     if (checkUninitVar) {
-                        checkUninitVar->uninitvarError(tok, c->varname);
+                        checkUninitVar->uninitvarError(tok, c->var->name());
                         break;
                     }
                 }
@@ -169,7 +183,7 @@ private:
         // bail out if first variable is a pointer
         for (it = checks.begin(); it != checks.end(); ++it) {
             UninitVar *c = dynamic_cast<UninitVar *>(*it);
-            if (c && c->varId == varid1 && c->pointer) {
+            if (c && c->varId == varid1 && c->var->isPointer() && !c->var->isArray()) {
                 bailOutVar(checks, varid1);
                 break;
             }
@@ -178,7 +192,7 @@ private:
         // bail out if second variable is a array/pointer
         for (it = checks.begin(); it != checks.end(); ++it) {
             UninitVar *c = dynamic_cast<UninitVar *>(*it);
-            if (c && c->varId == varid2 && (c->pointer || c->array)) {
+            if (c && c->varId == varid2 && (c->var->isPointer() || c->var->isArray())) {
                 bailOutVar(checks, varid2);
                 break;
             }
@@ -238,35 +252,35 @@ private:
                 // example: .. = var;
                 // it is ok to read the address of an uninitialized array.
                 // it is ok to read the address of an allocated pointer
-                if (mode == 0 && (c->array || (c->pointer && c->alloc)))
+                if (mode == 0 && (c->var->isArray() || (c->var->isPointer() && c->alloc)))
                     continue;
 
                 // mode 2 : reading array data with mem.. function. It's ok if the
-                //          array is not 0-terminated
+                //          array is not null-terminated
                 if (mode == 2 && c->strncpy_)
                     continue;
 
                 // mode 3 : bad usage of pointer. if it's not a pointer then the usage is ok.
                 // example: ptr->foo();
-                if (mode == 3 && !c->pointer)
+                if (mode == 3 && (!c->var->isPointer() || c->var->isArray()))
                     continue;
 
                 // mode 4 : using dead pointer is invalid.
-                if (mode == 4 && (!c->pointer || c->alloc))
+                if (mode == 4 && (!c->var->isPointer() || c->var->isArray() || c->alloc))
                     continue;
 
                 // mode 5 : reading uninitialized array or pointer is invalid.
-                if (mode == 5 && (!c->array && !c->pointer))
+                if (mode == 5 && (!c->var->isArray() && !c->var->isPointer()))
                     continue;
 
                 CheckUninitVar *checkUninitVar = dynamic_cast<CheckUninitVar *>(c->owner);
                 if (checkUninitVar) {
                     if (c->strncpy_ || c->memset_nonzero)
-                        checkUninitVar->uninitstringError(tok, c->varname, c->strncpy_);
-                    else if (c->pointer && c->alloc)
-                        checkUninitVar->uninitdataError(tok, c->varname);
+                        checkUninitVar->uninitstringError(tok, c->var->name(), c->strncpy_);
+                    else if (c->var->isPointer() && !c->var->isArray() && c->alloc)
+                        checkUninitVar->uninitdataError(tok, c->var->name());
                     else
-                        checkUninitVar->uninitvarError(tok, c->varname);
+                        checkUninitVar->uninitvarError(tok, c->var->name());
                     return true;
                 }
             }
@@ -297,7 +311,7 @@ private:
     }
 
     /**
-     * Reading array elements with a "mem.." function. It's ok if the array is not 0-terminated.
+     * Reading array elements with a "mem.." function. It's ok if the array is not null-terminated.
      * @param checks all available checks
      * @param tok variable token
      */
@@ -336,30 +350,6 @@ private:
         return use(checks, tok, 5);
     }
 
-    /** declaring a variable */
-    void declare(std::list<ExecutionPath *> &checks, const Token *vartok, const Token &tok, const bool p, const bool a) const {
-        if (vartok->varId() == 0)
-            return;
-
-        // Suppress warnings if variable in inner scope has same name as variable in outer scope
-        if (!tok.isStandardType()) {
-            std::set<unsigned int> dup;
-            for (std::list<ExecutionPath *>::const_iterator it = checks.begin(); it != checks.end(); ++it) {
-                UninitVar *c = dynamic_cast<UninitVar *>(*it);
-                if (c && c->varname == vartok->str() && c->varId != vartok->varId())
-                    dup.insert(c->varId);
-            }
-            if (!dup.empty()) {
-                for (std::set<unsigned int>::const_iterator it = dup.begin(); it != dup.end(); ++it)
-                    bailOutVar(checks, *it);
-                return;
-            }
-        }
-
-        if (a || p || tok.isStandardType())
-            checks.push_back(new UninitVar(owner, vartok->varId(), vartok->str(), p, a));
-    }
-
     /**
      * Parse right hand side expression in statement
      * @param tok2 start token of rhs
@@ -368,7 +358,7 @@ private:
     void parserhs(const Token *tok2, std::list<ExecutionPath *> &checks) const {
         // check variable usages in rhs/index
         while (NULL != (tok2 = tok2->next())) {
-            if (Token::Match(tok2, "[;)=?]"))
+            if (Token::Match(tok2, "[;)=]"))
                 break;
             if (Token::Match(tok2, "%var% ("))
                 break;
@@ -408,45 +398,41 @@ private:
     /** parse tokens. @sa ExecutionPath::parse */
     const Token *parse(const Token &tok, std::list<ExecutionPath *> &checks) const {
         // Variable declaration..
-        if (Token::Match(tok.previous(), "[;{}] %var%") && tok.str() != "return") {
-            if (Token::Match(&tok, "enum %type% {"))
-                return tok.linkAt(2);
-
-            const Token * vartok = &tok;
-            while (Token::Match(vartok, "const|struct"))
-                vartok = vartok->next();
-
-            if (Token::Match(vartok, "%type% *| %var% ;")) {
-                vartok = vartok->next();
-                const bool p(vartok->str() == "*");
-                if (p)
-                    vartok = vartok->next();
-                declare(checks, vartok, tok, p, false);
-                return vartok;
-            }
-
-            // Variable declaration for array..
-            if (Token::Match(vartok, "%type% %var% [") &&
-                vartok->isStandardType() &&
-                Token::simpleMatch(vartok->linkAt(2), "] ;")) {
-                vartok = vartok->next();
-                declare(checks, vartok, tok, false, true);
-                return vartok->next()->link();
-            }
-
-            // Template pointer variable..
-            if (Token::Match(vartok, "%type% ::|<")) {
-                while (Token::Match(vartok, "%type% ::"))
-                    vartok = vartok->tokAt(2);
-                if (Token::Match(vartok, "%type% < %type%")) {
-                    vartok = vartok->tokAt(3);
-                    while (vartok && (vartok->str() == "*" || vartok->isName()))
-                        vartok = vartok->next();
-                    if (Token::Match(vartok, "> * %var% ;")) {
-                        declare(checks, vartok->tokAt(2), tok, true, false);
-                        return vartok->tokAt(2);
-                    }
+        if (tok.varId() && Token::Match(&tok, "%var% [[;]")) {
+            const Variable* var2 = symbolDatabase->getVariableFromVarId(tok.varId());
+            if (var2 && var2->nameToken() == &tok && !var2->isStatic() && !var2->isExtern() && !var2->isConst()) {
+                if (tok.linkAt(1)) { // array
+                    const Token* endtok = tok.next();
+                    while (endtok->link())
+                        endtok = endtok->link()->next();
+                    if (endtok->str() != ";")
+                        return &tok;
                 }
+                const Scope* parent = var2->scope()->nestedIn;
+                while (parent) {
+                    for (std::list<Variable>::const_iterator j = parent->varlist.begin(); j != parent->varlist.end(); ++j) {
+                        if (j->name() == var2->name()) {
+                            ExecutionPath::bailOutVar(checks, j->varId()); // If there is a variable with the same name in other scopes, this might cause false positives, if there are unexpanded macros
+                            break;
+                        }
+                    }
+                    parent = parent->nestedIn;
+                }
+
+                if (var2->isPointer())
+                    checks.push_back(new UninitVar(owner, var2, symbolDatabase, isC));
+                else if (var2->typeEndToken()->str() != ">") {
+                    bool stdtype = false;  // TODO: change to isC to handle unknown types better
+                    for (const Token* tok2 = var2->typeStartToken(); tok2 != var2->nameToken(); tok2 = tok2->next()) {
+                        if (tok2->isStandardType()) {
+                            stdtype = true;
+                            break;
+                        }
+                    }
+                    if (stdtype && (!var2->isArray() || var2->nameToken()->linkAt(1)->strAt(1) == ";"))
+                        checks.push_back(new UninitVar(owner, var2, symbolDatabase, isC));
+                }
+                return &tok;
             }
         }
 
@@ -454,10 +440,16 @@ private:
             // is there assignment or ternary operator in the return statement?
             bool assignment = false;
             for (const Token *tok2 = tok.next(); tok2 && tok2->str() != ";"; tok2 = tok2->next()) {
-                if (tok2->str() == "=" || tok2->str() == ">>" || tok2->str() == "?" || Token::Match(tok2, "(|, &")) {
+                if (tok2->str() == "=" || (!isC && tok2->str() == ">>") || Token::Match(tok2, "(|, &")) {
                     assignment = true;
                     break;
                 }
+                if (Token::Match(tok2, "[(,] &| %var% [,)]")) {
+                    tok2 = tok2->next();
+                    if (!tok2->isName())
+                        tok2 = tok2->next();
+                    ExecutionPath::bailOutVar(checks, tok2->varId());
+                }
             }
 
             if (!assignment) {
@@ -482,7 +474,7 @@ private:
                 // variable is assigned like: "( %var% ) .. ="
                 if (Token::Match(tok.previous(), "( %var% )") && tok2 && tok2->str() == "=")
                     ExecutionPath::bailOutVar(checks, tok.varId());
-                else
+                else if (tok.strAt(-2) != ">" || !tok.linkAt(-2))
                     use(checks, &tok);
                 //use_array(checks, &tok);
                 return &tok;
@@ -490,6 +482,17 @@ private:
 
             // Used..
             if (Token::Match(tok.previous(), "[[(,+-*/|=] %var% ]|)|,|;|%op%")) {
+                // Taking address of array..
+                std::list<ExecutionPath *>::const_iterator it;
+                for (it = checks.begin(); it != checks.end(); ++it) {
+                    UninitVar *c = dynamic_cast<UninitVar *>(*it);
+                    if (c && c->varId == tok.varId()) {
+                        if (c->var->isArray())
+                            bailOutVar(checks, tok.varId());
+                        break;
+                    }
+                }
+
                 // initialize reference variable
                 if (Token::Match(tok.tokAt(-3), "& %var% ="))
                     bailOutVar(checks, tok.varId());
@@ -498,7 +501,7 @@ private:
                 return &tok;
             }
 
-            if (Token::Match(tok.previous(), "++|--") || Token::Match(tok.next(), "++|--")) {
+            if ((tok.previous() && tok.previous()->type() == Token::eIncDecOp) || (tok.next() && tok.next()->type() == Token::eIncDecOp)) {
                 use(checks, &tok);
                 return &tok;
             }
@@ -510,14 +513,26 @@ private:
                     }
                 } else {
                     const Token *tok2 = tok.next();
-                    if (tok2->str() == "[" && Token::simpleMatch(tok2->link(), "] =")) {
-                        if (use_dead_pointer(checks, &tok)) {
-                            return &tok;
-                        }
 
-                        parserhs(tok2, checks);
-                        tok2 = tok2->link()->next();
+                    if (tok2->str() == "[") {
+                        const Token *tok3 = tok2->link();
+                        while (Token::simpleMatch(tok3, "] ["))
+                            tok3 = tok3->next()->link();
+
+                        // Possible initialization
+                        if (Token::simpleMatch(tok3, "] >>"))
+                            return &tok;
+
+                        if (Token::simpleMatch(tok3, "] =")) {
+                            if (use_dead_pointer(checks, &tok)) {
+                                return &tok;
+                            }
+
+                            parserhs(tok2, checks);
+                            tok2 = tok3->next();
+                        }
                     }
+
                     parserhs(tok2, checks);
                 }
 
@@ -559,7 +574,7 @@ private:
                     return tok.tokAt(3);
             }
 
-            else if (Token::Match(tok.previous(), "<<|>>") || Token::simpleMatch(tok.next(), "=")) {
+            else if ((!isC && Token::Match(tok.previous(), "<<|>>")) || Token::simpleMatch(tok.next(), "=")) {
                 // TODO: Don't bail out for "<<" and ">>" if these are
                 // just computations
                 ExecutionPath::bailOutVar(checks, tok.varId());
@@ -596,9 +611,9 @@ private:
 
             // parse usage..
             {
-                std::list<const Token *> var;
-                CheckNullPointer::parseFunctionCall(tok, var, 1);
-                for (std::list<const Token *>::const_iterator it = var.begin(); it != var.end(); ++it) {
+                std::list<const Token *> var1;
+                CheckNullPointer::parseFunctionCall(tok, var1, 1);
+                for (std::list<const Token *>::const_iterator it = var1.begin(); it != var1.end(); ++it) {
                     // does iterator point at first function parameter?
                     const bool firstPar(*it == tok.tokAt(2));
 
@@ -620,12 +635,12 @@ private:
                 std::list<const Token *> var2;
                 CheckNullPointer::parseFunctionCall(tok, var2, 0);
                 for (std::list<const Token *>::const_iterator it = var2.begin(); it != var2.end(); ++it) {
-                    if (std::find(var.begin(), var.end(), *it) == var.end())
+                    if (std::find(var1.begin(), var1.end(), *it) == var1.end())
                         use_dead_pointer(checks, *it);
                 }
             }
 
-            // strncpy doesn't 0-terminate first parameter
+            // strncpy doesn't null-terminate first parameter
             if (Token::Match(&tok, "strncpy ( %var% ,")) {
                 if (Token::Match(tok.tokAt(4), "%str% ,")) {
                     if (Token::Match(tok.tokAt(6), "%num% )")) {
@@ -673,7 +688,7 @@ private:
                 }
 
                 // ticket #2367 : unexpanded macro that uses sizeof|typeof?
-                else if (Token::Match(tok2, "%type% (") && CheckNullPointer::isUpper(tok2->str())) {
+                else if (Token::Match(tok2, "%type% (") && tok2->isUpperCaseName()) {
                     tok2 = tok2->next()->link();
                     if (!tok2)
                         break;
@@ -692,7 +707,7 @@ private:
 
                         functionCall = functionCall ? functionCall->previous() : 0;
                         if (functionCall) {
-                            if (functionCall->isName() && !CheckNullPointer::isUpper(functionCall->str()) && use_dead_pointer(checks, tok2))
+                            if (functionCall->isName() && !functionCall->isUpperCaseName() && use_dead_pointer(checks, tok2))
                                 ExecutionPath::bailOutVar(checks, tok2->varId());
                         }
                     }
@@ -707,7 +722,7 @@ private:
                         for (std::list<ExecutionPath *>::const_iterator it = checks.begin(); it != checks.end(); ++it) {
                             if ((*it)->varId == tok2->varId()) {
                                 const UninitVar *c = dynamic_cast<const UninitVar *>(*it);
-                                if (c && (c->array || (c->pointer && c->alloc)))
+                                if (c && (c->var->isArray() || (c->var->isPointer() && c->alloc)))
                                     bailouts.insert(tok2->varId());
                                 break;
                             }
@@ -724,18 +739,9 @@ private:
         if (Token::Match(&tok, "( * %var% ) (") ||
             (Token::Match(&tok, "( *| %var% .|::") && Token::Match(tok.link()->tokAt(-2), ".|:: %var% ) ("))) {
             // is the variable passed as a parameter to some function?
-            unsigned int parlevel = 0;
-            for (const Token *tok2 = tok.link()->next(); tok2; tok2 = tok2->next()) {
-                if (tok2->str() == "(")
-                    ++parlevel;
-
-                else if (tok2->str() == ")") {
-                    if (parlevel <= 1)
-                        break;
-                    --parlevel;
-                }
-
-                else if (tok2->varId()) {
+            const Token *tok2 = tok.link()->next();
+            for (const Token* const end2 = tok2->link(); tok2 != end2; tok2 = tok2->next()) {
+                if (tok2->varId()) {
                     // it is possible that the variable is initialized here
                     ExecutionPath::bailOutVar(checks, tok2->varId());
                 }
@@ -864,14 +870,14 @@ private:
         if (tok.varId() && Token::Match(&tok, "%var% <|<=|==|!=|)"))
             use(checks, &tok);
 
-        else if (Token::Match(&tok, "!| %var% ["))
+        else if (Token::Match(&tok, "!| %var% [") && !Token::simpleMatch(skipBrackets(tok.next()), "="))
             use_array_or_pointer_data(checks, tok.str() == "!" ? tok.next() : &tok);
 
         else if (Token::Match(&tok, "!| %var% (")) {
             const Token * const ftok = (tok.str() == "!") ? tok.next() : &tok;
-            std::list<const Token *> var;
-            CheckNullPointer::parseFunctionCall(*ftok, var, 1);
-            for (std::list<const Token *>::const_iterator it = var.begin(); it != var.end(); ++it) {
+            std::list<const Token *> var1;
+            CheckNullPointer::parseFunctionCall(*ftok, var1, 1);
+            for (std::list<const Token *>::const_iterator it = var1.begin(); it != var1.end(); ++it) {
                 // is function memset/memcpy/etc?
                 if (ftok->str().compare(0,3,"mem") == 0)
                     use_array_mem(checks, *it);
@@ -894,15 +900,9 @@ private:
                 return;
             if (Token::simpleMatch(tok, "if (")) {
                 // bail out all variables that are used in the condition
-                unsigned int parlevel = 0;
-                for (const Token *tok2 = tok->tokAt(2); tok2; tok2 = tok2->next()) {
-                    if (tok2->str() == "(")
-                        ++parlevel;
-                    else if (tok2->str() == ")") {
-                        if (parlevel == 0)
-                            break;
-                        --parlevel;
-                    } else if (tok2->varId())
+                const Token* const end2 = tok->linkAt(1);
+                for (const Token *tok2 = tok->tokAt(2); tok2 != end2; tok2 = tok2->next()) {
+                    if (tok2->varId())
                         ExecutionPath::bailOutVar(checks, tok2->varId());
                 }
             }
@@ -953,8 +953,8 @@ public:
                             } else if (indentlevel == 0 && tok3->str() == ";")
                                 break;
                             else if (indentlevel >= 1 && tok3->varId() == varid) {
-                                if (Token::Match(tok3->previous(), "++|--") ||
-                                    Token::Match(tok3->next(), "++|--")) {
+                                if (tok3->previous()->type() == Token::eIncDecOp ||
+                                    tok3->next()->type() == Token::eIncDecOp) {
                                     r = true;
                                 }
 
@@ -1031,7 +1031,7 @@ void CheckUninitVar::executionPaths()
         if (_settings->_jobs == 1)
             UninitVar::analyseFunctions(_tokenizer->tokens(), UninitVar::uvarFunctions);
 
-        UninitVar c(this);
+        UninitVar c(this, _tokenizer->getSymbolDatabase(), _tokenizer->isC());
         checkExecutionPaths(_tokenizer->getSymbolDatabase(), &c);
     }
 }
@@ -1046,39 +1046,50 @@ void CheckUninitVar::check()
     for (func_scope = symbolDatabase->scopeList.begin(); func_scope != symbolDatabase->scopeList.end(); ++func_scope) {
         // only check functions
         if (func_scope->type == Scope::eFunction) {
-            for (const Token *tok = func_scope->classStart; tok && tok != func_scope->classEnd; tok = tok->next()) {
-
-                if (Token::Match(tok, "[;{}] %type% !!;")) {
-                    bool stdtype = false;
-                    bool pointer = false;
-                    tok = tok->next();
-                    while (tok->isName() || tok->str() == "*") {
-                        if (tok->isStandardType())
-                            stdtype = true;
-                        else if (tok->str() == "*")
-                            pointer = true;
-                        else if (Token::Match(tok, "struct %type%"))
-                            tok = tok->next();
-                        else {
-                            if (tok->isName() && tok->next()->str() == ";" &&
-                                (stdtype || pointer)) {
-                                checkScopeForVariable(tok->next(), tok->varId(), pointer, NULL);
-                            }
-                            break;
-                        }
-                        tok = tok->next();
-                    }
-                }
-            }
+            checkScope(&*func_scope);
         }
     }
 }
 
-bool CheckUninitVar::checkScopeForVariable(const Token *tok, const unsigned int varid, bool ispointer, bool * const possibleInit)
+void CheckUninitVar::checkScope(const Scope* scope)
 {
-    if (varid == 0)
-        return false;
+    for (std::list<Variable>::const_iterator i = scope->varlist.begin(); i != scope->varlist.end(); ++i) {
+        if ((i->type() && !i->isPointer()) || i->isStatic() || i->isExtern() || i->isConst() || i->isArray() || i->isReference())
+            continue;
+        if (i->nameToken()->strAt(1) == "(")
+            continue;
 
+        bool forHead = false; // Don't check variables declared in header of a for loop
+        for (const Token* tok = i->typeStartToken(); tok; tok = tok->previous()) {
+            if (tok->str() == "(") {
+                forHead = true;
+                break;
+            } else if (tok->str() == "{" || tok->str() == ";" || tok->str() == "}")
+                break;
+        }
+        if (forHead)
+            continue;
+
+        bool stdtype = _tokenizer->isC();
+        const Token* tok = i->typeStartToken();
+        for (; tok && tok->str() != ";" && tok->str() != "<"; tok = tok->next()) {
+            if (tok->isStandardType())
+                stdtype = true;
+        }
+        while (tok && tok->str() != ";")
+            tok = tok->next();
+        if (stdtype || i->isPointer())
+            checkScopeForVariable(tok, *i, NULL);
+    }
+
+    for (std::list<Scope*>::const_iterator i = scope->nestedList.begin(); i != scope->nestedList.end(); ++i) {
+        if (!(*i)->isClassOrStruct())
+            checkScope(*i);
+    }
+}
+
+bool CheckUninitVar::checkScopeForVariable(const Token *tok, const Variable& var, bool * const possibleInit)
+{
     const bool suppressErrors(possibleInit && *possibleInit);
 
     if (possibleInit)
@@ -1106,7 +1117,7 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const unsigned int
 
         // Unconditional inner scope..
         if (tok->str() == "{" && Token::Match(tok->previous(), "[;{}]")) {
-            if (checkScopeForVariable(tok->next(), varid, ispointer, possibleInit))
+            if (checkScopeForVariable(tok->next(), var, possibleInit))
                 return true;
             tok = tok->link();
             continue;
@@ -1119,7 +1130,7 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const unsigned int
         // Inner scope..
         if (Token::simpleMatch(tok, "if (")) {
             // initialization / usage in condition..
-            if (checkIfForWhileHead(tok->next(), varid, ispointer, suppressErrors, bool(number_of_if == 0)))
+            if (checkIfForWhileHead(tok->next(), var, suppressErrors, bool(number_of_if == 0)))
                 return true;
 
             // checking if a not-zero variable is zero => bail out
@@ -1133,7 +1144,7 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const unsigned int
                 break;
             if (tok->str() == "{") {
                 bool possibleInitIf(number_of_if > 0 || suppressErrors);
-                const bool initif = checkScopeForVariable(tok->next(), varid, ispointer, &possibleInitIf);
+                const bool initif = checkScopeForVariable(tok->next(), var, &possibleInitIf);
 
                 // goto the }
                 tok = tok->link();
@@ -1149,7 +1160,7 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const unsigned int
                     tok = tok->tokAt(2);
 
                     bool possibleInitElse(number_of_if > 0 || suppressErrors);
-                    const bool initelse = checkScopeForVariable(tok->next(), varid, ispointer, &possibleInitElse);
+                    const bool initelse = checkScopeForVariable(tok->next(), var, &possibleInitElse);
 
                     // goto the }
                     tok = tok->link();
@@ -1169,7 +1180,7 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const unsigned int
             const Token *end = tok->next()->link();
 
             // If address of variable is taken in the block then bail out
-            if (Token::findmatch(tok->tokAt(2), "& %varid%", end, varid))
+            if (Token::findmatch(tok->tokAt(2), "& %varid%", end, var.varId()))
                 return true;
 
             // Skip block
@@ -1184,7 +1195,7 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const unsigned int
         // for..
         if (Token::simpleMatch(tok, "for (")) {
             // is variable initialized in for-head (don't report errors yet)?
-            if (checkIfForWhileHead(tok->next(), varid, ispointer, true, false))
+            if (checkIfForWhileHead(tok->next(), var, true, false))
                 return true;
 
             // goto the {
@@ -1192,7 +1203,7 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const unsigned int
 
             if (tok2 && tok2->str() == "{") {
                 bool possibleinit = true;
-                bool init = checkScopeForVariable(tok2->next(), varid, ispointer, &possibleinit);
+                bool init = checkScopeForVariable(tok2->next(), var, &possibleinit);
 
                 // variable is initialized in the loop..
                 if (possibleinit || init)
@@ -1200,7 +1211,7 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const unsigned int
 
                 // is variable used in for-head?
                 if (!suppressErrors) {
-                    checkIfForWhileHead(tok->next(), varid, ispointer, false, bool(number_of_if == 0));
+                    checkIfForWhileHead(tok->next(), var, false, bool(number_of_if == 0));
                 }
             }
         }
@@ -1221,9 +1232,9 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const unsigned int
             return true;
 
         // variable is seen..
-        if (tok && tok->varId() == varid) {
+        if (tok->varId() == var.varId()) {
             // Use variable
-            if (!suppressErrors && isVariableUsage(tok, ispointer))
+            if (!suppressErrors && isVariableUsage(tok, var.isPointer()))
                 uninitvarError(tok, tok->str());
 
             else
@@ -1235,12 +1246,12 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const unsigned int
     return ret;
 }
 
-bool CheckUninitVar::checkIfForWhileHead(const Token *startparanthesis, unsigned int varid, bool ispointer, bool suppressErrors, bool isuninit)
+bool CheckUninitVar::checkIfForWhileHead(const Token *startparanthesis, const Variable& var, bool suppressErrors, bool isuninit)
 {
     const Token * const endpar = startparanthesis->link();
     for (const Token *tok = startparanthesis->next(); tok && tok != endpar; tok = tok->next()) {
-        if (tok->varId() == varid) {
-            if (isVariableUsage(tok, ispointer)) {
+        if (tok->varId() == var.varId()) {
+            if (isVariableUsage(tok, var.isPointer())) {
                 if (!suppressErrors)
                     uninitvarError(tok, tok->str());
                 else
@@ -1267,6 +1278,27 @@ bool CheckUninitVar::isVariableUsage(const Token *vartok, bool pointer) const
             return false;
         }
 
+        // is there something like: ; "*((&var ..expr.. ="  => the variable is assigned
+        if (vartok->previous()->str() == "&") {
+            const Token *tok2 = vartok->tokAt(-2);
+            if (Token::simpleMatch(tok2,")"))
+                tok2 = tok2->link()->previous();
+            while (tok2 && tok2->str() == "(")
+                tok2 = tok2->previous();
+            while (tok2 && tok2->str() == "*")
+                tok2 = tok2->previous();
+            if (Token::Match(tok2, "[;{}] *")) {
+                // there is some such code before vartok: "[*]+ [(]* &"
+                // determine if there is a = after vartok
+                for (tok2 = vartok; tok2; tok2 = tok2->next()) {
+                    if (Token::Match(tok2, "[;{}]"))
+                        break;
+                    if (tok2->str() == "=")
+                        return false;
+                }
+            }
+        }
+
         if (vartok->previous()->str() != "&" || !Token::Match(vartok->tokAt(-2), "[(,=?:]")) {
             return true;
         }
@@ -1284,6 +1316,15 @@ bool CheckUninitVar::isVariableUsage(const Token *vartok, bool pointer) const
             return true;
     }
 
+    if (_tokenizer->isCPP() && Token::Match(vartok->next(), "<<|>>")) {
+        // Is variable a known POD type then this is a variable usage,
+        // otherwise we assume it's not.
+        const Variable *var = _tokenizer->getSymbolDatabase()->getVariableFromVarId(vartok->varId());
+        if (var && var->typeStartToken()->isStandardType())
+            return true;
+        return false;
+    }
+
     if (Token::Match(vartok->next(), "++|--|%op%"))
         return true;
 
@@ -1296,12 +1337,12 @@ bool CheckUninitVar::isVariableUsage(const Token *vartok, bool pointer) const
 
 void CheckUninitVar::uninitstringError(const Token *tok, const std::string &varname, bool strncpy_)
 {
-    reportError(tok, Severity::error, "uninitstring", "Dangerous usage of '" + varname + "'" + (strncpy_ ? " (strncpy doesn't always 0-terminate it)" : " (not 0-terminated)"));
+    reportError(tok, Severity::error, "uninitstring", "Dangerous usage of '" + varname + "'" + (strncpy_ ? " (strncpy doesn't always null-terminate it)." : " (not null-terminated)."));
 }
 
 void CheckUninitVar::uninitdataError(const Token *tok, const std::string &varname)
 {
-    reportError(tok, Severity::error, "uninitdata", "Data is allocated but not initialized: " + varname);
+    reportError(tok, Severity::error, "uninitdata", "Memory is allocated but not initialized: " + varname);
 }
 
 void CheckUninitVar::uninitvarError(const Token *tok, const std::string &varname)
